# This file is part of monday-client.
#
# Copyright (C) 2024 Leet Cyber Security <https://leetcybersecurity.com/>
#
# monday-client is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# monday-client is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with monday-client. If not, see <https://www.gnu.org/licenses/>.

"""Tests for monday-client package exports."""

import pytest

pytestmark = pytest.mark.unit

import monday


def test_package_has_version_and_exports():
    """Test version string and key package exports exist."""
    assert isinstance(monday.__version__, str)
    assert monday.__version__
    # spot check a few exports
    assert hasattr(monday, 'MondayClient')
    assert hasattr(monday, 'BoardFields')
    assert hasattr(monday, 'Fields')
